///////////////////////////////////////////////////////////////////////////
//
// LE-650H2 Sample program : LEU_GUISample
//                           ( GUI & Trigger/Filter test)
//
//////////////////////////////////////////////////////////////////////////

#include "LEU_GUISample.h"
#pragma comment(lib, "comctl32.lib ")

char * griddata[GRID_LINES][GRID_COLS] = {NULL}; // buffer for the virtual listview

/// add a message to the status log window ///
int StatusMsg(const char * mstr){

	int nl = SendMessage(hEditSt, EM_GETLINECOUNT , 0 , 0);
	int pos0, pos1;

	while(nl >= (ST_MAX_LINE)){ // clear the first line
		pos0 = SendMessage(hEditSt, EM_LINEINDEX,(WPARAM)0, (LPARAM)0);
		pos1 = SendMessage(hEditSt, EM_LINEINDEX,(WPARAM)1, (LPARAM)0);
		SendMessage(hEditSt, EM_SETSEL, (WPARAM)pos0, (LPARAM)pos1);
		SendMessage(hEditSt, EM_REPLACESEL, FALSE, (LPARAM)"");
		nl = SendMessage(hEditSt, EM_GETLINECOUNT , 0 , 0);
	}

	// add the last line
	int len = SendMessage(hEditSt, WM_GETTEXTLENGTH, (WPARAM)0,(LPARAM)0);
	SendMessage(hEditSt, EM_SETSEL,(WPARAM)len,(LPARAM)len);
	SendMessage(hEditSt, EM_REPLACESEL, FALSE, (LPARAM)mstr);
	// scroll
	SendMessage(hEditSt, WM_VSCROLL, (WPARAM)SB_BOTTOM, (LPARAM)0);
	return len;
}

/// add a message to the status log window via sprintf ///
int StatusMsgSP(const char * fmt, ...){
	char buf[200];
	va_list argp;
	va_start( argp, fmt );
	vsnprintf_s( buf, 200, _TRUNCATE, fmt, argp );
	va_end(argp);
	return StatusMsg(buf);
}

///////////////////////////////////////////////////////////
/// Run status functions 
///////////////////////////////////////////////////////////

void NoticeUnconnected(){
	SetWindowText(hTextCon,"Unconnected");
	SetWindowText(hTextRun,"Stopping");
	EnableWindow(hEditSer, TRUE);
	EnableWindow(hEditTrg1, TRUE);
	EnableWindow(hEditTrg2, TRUE);
	EnableWindow(hBtnSer, TRUE);
	EnableWindow(hBtnDcon, FALSE);
	EnableWindow(hBtnRun, FALSE);
	EnableWindow(hBtnStop, FALSE);
	EnableWindow(hBtnSave, TRUE);
	EnableWindow(hChkSOF, TRUE);
	EnableWindow(hChkPING, TRUE);
	EnableWindow(hChkINNAK, TRUE);
	EnableWindow(hChkSETUP, TRUE);
}
void NoticeStopping(){
	SetWindowText(hTextCon,"Connected");
	SetWindowText(hTextRun,"Stopping");
	EnableWindow(hEditSer, FALSE);
	EnableWindow(hEditTrg1, TRUE);
	EnableWindow(hEditTrg2, TRUE);
	EnableWindow(hBtnSer, FALSE);
	EnableWindow(hBtnDcon, TRUE);
	EnableWindow(hBtnRun, TRUE);
	EnableWindow(hBtnStop, FALSE);
	EnableWindow(hBtnSave, TRUE);
	EnableWindow(hChkSOF, TRUE);
	EnableWindow(hChkPING, TRUE);
	EnableWindow(hChkINNAK, TRUE);
	EnableWindow(hChkSETUP, TRUE);
}
void NoticeRunning(){
	SetWindowText(hTextCon,"Connected");
	SetWindowText(hTextRun,"Running");
	EnableWindow(hEditSer, FALSE);
	EnableWindow(hEditTrg1, FALSE);
	EnableWindow(hEditTrg2, FALSE);
	EnableWindow(hBtnSer, FALSE);
	EnableWindow(hBtnDcon, FALSE);
	EnableWindow(hBtnRun, FALSE);
	EnableWindow(hBtnStop, TRUE);
	EnableWindow(hBtnSave, FALSE);
	EnableWindow(hChkSOF, FALSE);
	EnableWindow(hChkPING, FALSE);
	EnableWindow(hChkINNAK, FALSE);
	EnableWindow(hChkSETUP, FALSE);
}

int CheckLE(){
	int stat = LE650->CheckState();
	if(stat == LEU_STATE_MEASURING){
		NoticeRunning();
		return stat;
	}
	if(stat > 999) NoticeUnconnected(); 
	else {
		NoticeStopping();
	}
	KillTimer(hWndMain, ID_TIMER); // stop read/update
	return stat;
}

///////////////////////////////////////////////////////////
/// Run control functions 
///////////////////////////////////////////////////////////

int OnConnect(){
	char buf[100];
	GetWindowText(hEditSer, buf, 100);
	// input form is empty or default -> try connection without serial //
	if(buf[0]=='\r'||buf[0]=='\0'||strcmp(buf,"<SERIAL>")==0){
		StatusMsg("LE connect without Serial ... ");
		return LE650->ConnectNoSerial();
	}
	else{
		StatusMsgSP("LE connect Serial = %s  ... ",buf); 
		return LE650->Connect(buf);
	}	
	return LEU_FAIL;
}

int OnDisConnect(){

	StatusMsg("LE Disconnect");

	return LE650->DisConnect();
}

int OnTrgSet(){
	char sdata[9], sdata2[9];
	int len = 0, len2 = 0;
	len = GetWindowText(hEditTrg1, sdata, 9);
	len2 = GetWindowText(hEditTrg2, sdata2, 9);
	StatusMsgSP("First trigger = %s  : %d  bytes\r\n", sdata, len);
	StatusMsgSP("Stop  trigger = %s  : %d  bytes\r\n", sdata2, len2); 

	return LE650->SetMyTrig(len, (uint8_t*)sdata, len2, (uint8_t*)sdata2);
}

int OnFilterSet(){
	int err = 0;
	uint16_t flt = 0x0;
	if( SendMessage(hChkSOF, BM_GETCHECK,0,0) == BST_CHECKED) flt |= LEU_FILTER_SOF;
	if( SendMessage(hChkPING, BM_GETCHECK,0,0) == BST_CHECKED) flt |= LEU_FILTER_PING;
	if( SendMessage(hChkINNAK, BM_GETCHECK,0,0) == BST_CHECKED) flt |= LEU_FILTER_IN_NAK;

	StatusMsg("Setting Hardware Filters...");
	if(LE650->SetHardFilter(flt)==LEU_FAIL){
		StatusMsg("Failed\r\n");
		err = LEU_FAIL;
	}
	else StatusMsg("Success\r\n");

	if( SendMessage(hChkSETUP, BM_GETCHECK,0,0) == BST_CHECKED) LE650->SetSoftFilter(MYFILTER_ONLYSETUP);
	else LE650->SetSoftFilter(MYFILTER_NONE);

	return err;
}

int OnRun(){
	ListView_DeleteAllItems(hGrid); 	// clear all data on the grid
	for(int i=0;i<GRID_LINES;i++){
		for(int j=0;j<GRID_COLS;j++){
			if(griddata[i][j]!=NULL){
				free(griddata[i][j]);  // delete old strings
				griddata[i][j]=NULL;
			}
		}
	}

	StatusMsg("LE Run start ..."); 
	return LE650->Run();
}

int OnStop(){
	StatusMsg("LE Run manual stop ..."); 
	return LE650->Stop();
}

int OnSave(OPENFILENAME ofn){
	int rt = GetSaveFileName(&ofn);
	if(!rt) return 0; // user cancel
	if(LE650->CheckState() == LEU_STATE_MEASURING){
		StatusMsg("Error : Please stop measurement before exporting data! \r\n");
		return 0;
	}

	std::ofstream ofs(ofn.lpstrFile, std::ios::trunc);
	StatusMsgSP("Save File : %s \r\n",ofn.lpstrFile);

	int nline = SendMessage(hGrid, LVM_GETITEMCOUNT, 0, 0);
	for(int i=0;i<nline;i++){
		for(int j=0; j<GRID_COLS; j++){
			if(j) ofs << ",";
			if(griddata[i][j]!=NULL)
				ofs << griddata[i][j];
		}
		ofs << std::endl;
	}
	ofs.close();
	return rt;

}

///////////////////////////////////////////////////////////
/// Timer event
/// called periodically while running
/// put user reading/analysis codes here
///////////////////////////////////////////////////////////

int OnTimer(){

	int tflag, disp;
	int rd;
	int stopflag = 0;
	int nline = SendMessage(hGrid, LVM_GETITEMCOUNT, 0, 0);
	int nline0 = nline;
	int rcount = 0;
	int zerocount = 0;

	if(nline >= GRID_LINES) return 2; // exceeds line limit

	while(!stopflag){
		while(true){
			rd = LE650->ReadData(); // get a packet
			if( rd != LEU_FAIL) break; // data exists
			zerocount ++;
			Sleep(1); // try 1ms x 10 times
			if(zerocount == 10){ // no data obtained
				if(CheckLE() != LEU_STATE_MEASURING) stopflag = 4; // maybe trigger stop
				else stopflag = 3; // just quit this timer event 
				break;
			}
		}
		if(rd == 2) continue; // skip SOF

		rcount ++;
		if(rcount > 50000) stopflag = 3; // prevent freezing, just quit this timer event

		tflag = LE650->AppendData(); // packet -> transaction
		if(tflag != 0){
			while(disp = LE650->GetNextTrans()){ // readable transaction/event exists.
				if((nline == 0 ) || !(disp& DISP_NOLINE)){
					nline ++;
				}
				WriteLine(nline-1, disp); // display
				if(nline == GRID_LINES){
					stopflag = 2; // stop run
					break;
				}
			}
		}
	}

	if(LE650->EndRead()) stopflag = 1; // check timeout

	char buf[100];
	// display SOF
	sprintf_s(buf,100,"LAST SOF : 0x%04x",LE650->last_sof);
	SetWindowText(hTextSOF, buf);
	// display time
	sprintf_s(buf,100,"LAST TIMESTAMP : %.4f sec.",LE650->GetLastTime());
	SetWindowText(hTextTime, buf);

	// window update
	if(nline != nline0){
		SendMessage(hGrid, LVM_SETITEMCOUNT, nline, LVSICF_NOINVALIDATEALL);
		SendMessage(hGrid, LVM_ENSUREVISIBLE, nline-1, FALSE);
		UpdateWindow(hGrid);
	}

	switch (stopflag){
	case 1:
		LE650->Stop();
		while(LE650->ReadData()!=LEU_FAIL); // read all
		StatusMsgSP("LE Run auto stop : Time out = %d msec. x %d \r\n", LEMES_REP, LERUN_TIMES);
		CheckLE();
		return 1;
	case 2:
		LE650->Stop();
		while(LE650->ReadData()!=LEU_FAIL); // read all
		StatusMsgSP("LE Run auto stop : MAX Output Lines = %d \r\n", GRID_LINES);
		CheckLE();
		return 2;
	case 3: // wait next timer
		break;
	case 4:
		// measurement already stopped
		while(LE650->ReadData()!=LEU_FAIL); // read all
		StatusMsgSP("LE Run auto stop : Trigger or Error \r\n");
		CheckLE();
		break;
	default:
		break;
	}
	return 0;
}

///////////////////////////////////////////////////////////
//
// Listview functions
//
///////////////////////////////////////////////////////////

void AddColumn(HWND _wnd, int wid, char * label, int id){
	LVCOLUMN lvcol;
	lvcol.mask=LVCF_TEXT | LVCF_SUBITEM | LVCF_WIDTH | LVCF_FMT;
	lvcol.fmt=LVCFMT_LEFT;
	lvcol.cx=wid;
	lvcol.pszText = label;
	SendMessage(_wnd ,LVM_INSERTCOLUMN, id ,(WPARAM)&lvcol);
}

void AddItem(HWND _wnd, int line, int col, const char * str){
	if(line >= GRID_LINES) return;
	char *tmp = NULL;
	if(griddata[line][col]!=NULL) tmp = griddata[line][col]; 
	griddata[line][col] = _strdup(str); // copy
	if(tmp!=NULL) free(tmp); // free old one
}

int WriteLine(int lineno, int disp){
	if(!disp) return -1; // no valid entries
	if(lineno >= GRID_LINES) return -1;

	if(disp & DISP_NTIMES){ // only updating dup. numbers
		AddItem(hGrid, lineno, 9, LE650->GetDispStr(DISP_NTIMES));
		return 0;
	}

	if(disp & DISP_PID)AddItem(hGrid, lineno, 0,  LE650->GetDispStr(DISP_PID));
	if(disp & DISP_VALUE) AddItem(hGrid, lineno, 1, LE650->GetDispStr(DISP_VALUE));
	if(disp & DISP_ADDR) AddItem(hGrid, lineno, 2, LE650->GetDispStr(DISP_ADDR));
	if(disp & DISP_ENDP) AddItem(hGrid, lineno, 3, LE650->GetDispStr(DISP_ENDP));
	if(disp & DISP_DATA){
		AddItem(hGrid, lineno, 4, LE650->GetDispStr(DISP_DATA_ID));
		AddItem(hGrid, lineno, 5, LE650->GetDispStr(DISP_DATA_LEN));
		AddItem(hGrid, lineno, 6, LE650->GetDispStr(DISP_DATA));
	}
	if(disp & DISP_HAND) AddItem(hGrid, lineno, 7, LE650->GetDispStr(DISP_HAND));
	if(disp & DISP_SPD) AddItem(hGrid, lineno, 8, LE650->GetDispStr(DISP_SPD));

	return 0;
}

///////////////////////////////////////////////////////////
//
// Main window callback
// 
///////////////////////////////////////////////////////////

LRESULT CALLBACK WndProc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp) {

	DWORD id = GetDlgCtrlID((HWND)lp);
	int ii;
	char buf[100];
	LOGFONT fsz12 = {12};
	static HFONT font12 = CreateFontIndirect(&fsz12); // 12pt font
	LOGFONT fsz24 = {24};
	static HFONT font24 = CreateFontIndirect(&fsz24); // 24pt font
	static int reading = 0;
	static HBRUSH btnBRS = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
	DWORD lvstyle;

	static OPENFILENAME ofn ={0};
	static char fname[MAX_PATH];

	switch(msg){
	case WM_DESTROY:
		DeleteObject(font12);
		DeleteObject(font24);
		DeleteObject(btnBRS);
		PostQuitMessage(0);
		return 0;

	case WM_COMMAND:
		switch(id){
		case ID_BTN_SER: // get serial no. & connect
			if(OnConnect() == LEU_FAIL){
				StatusMsgSP("Failed : %s\r\n",LE650->GetErrStatus());
			}
			else{
				StatusMsg("Success\r\n");
				NoticeStopping();
				if( LE650->InitSettings() == LEU_FAIL){
					StatusMsgSP("Trigger&Filter Init Failed : %s\r\n",LE650->GetErrStatus());
					CheckLE();
				}
			}
			break;
		case ID_BTN_DCON:

			if(OnDisConnect() == LEU_FAIL){
				StatusMsgSP("Failed : %s\r\n",LE650->GetErrStatus());
				CheckLE();
			}
			else{
				StatusMsg("Success\r\n");
				NoticeUnconnected();
			}

			break;
		case ID_BTN_RUN: // set trigger & start run
			if(OnTrgSet() == LEU_FAIL){
				StatusMsgSP("Trigger Setting Failed : %s\r\n",LE650->GetErrStatus());
				CheckLE();
			}
			if(OnFilterSet() !=0 ){
				StatusMsgSP("%s\r\n",LE650->GetErrStatus());
			}
			if(OnRun() == LEU_FAIL){
				StatusMsgSP("Failed : %s\r\n",LE650->GetErrStatus());
				CheckLE();
			}
			else{
				StatusMsg("Success\r\n");
				NoticeRunning();
				SetTimer(hWndMain, ID_TIMER, LEMES_REP, NULL); // periodical read start
			}
			break;
		case ID_BTN_STOP:
			if(OnStop() == LEU_FAIL){
				StatusMsgSP("Failed : %s\r\n",LE650->GetErrStatus());
				CheckLE();
			}
			else{
				StatusMsg("Success\r\n");
				NoticeStopping();
				KillTimer(hWndMain, ID_TIMER);
			}
			break;
		case ID_BTN_SAVE:
			OnSave(ofn);
			break;
		}
		break;
	case WM_CTLCOLORSTATIC:
		if(id == ID_TXT_CON){
			COLORREF clr = RGB(255,0,0);
			GetWindowText(hTextCon, buf, 100);
			if(strcmp(buf,"Connected")==0) clr = RGB(0,255,0);
			SetTextColor((HDC)wp, clr);
			SetBkMode((HDC)wp, OPAQUE);
			SetBkColor((HDC)wp, GetSysColor(COLOR_BTNFACE));
			return (INT_PTR) btnBRS;
		}
		else if(id == ID_TXT_RUN){
			COLORREF clr = RGB(255,0,0);
			GetWindowText(hTextRun, buf, 100);
			if(strcmp(buf,"Running")==0) clr = RGB(0,255,0);
			SetTextColor((HDC)wp, clr);
			SetBkMode((HDC)wp, OPAQUE);
			SetBkColor((HDC)wp, GetSysColor(COLOR_BTNFACE));
			return (INT_PTR) btnBRS;
		}
		break;
	case WM_TIMER:
		if(!reading){ // prevent duplicater call while reading
			reading = 1;
			OnTimer(); // call timer event
			reading = 0;
		}
		break;
	case WM_NOTIFY: // update virtual listview
		if(((LPNMHDR)lp)->code == LVN_GETDISPINFO){
			NMLVDISPINFO *Dinfo = (NMLVDISPINFO *)lp;
			Dinfo->item.pszText = griddata[Dinfo->item.iItem][Dinfo->item.iSubItem];
			return 0;
		}
		break;
	case WM_CREATE: // init sub-windows
		hRcTxlib = LoadLibrary("RICHED20.DLL");

		// status log window : richedit
		hEditSt = CreateWindowEx(WS_EX_CLIENTEDGE,
			"RICHEDIT20A",
			"Status Message\n",
			WS_CHILD | WS_VISIBLE | WS_BORDER | ES_MULTILINE | ES_LEFT | 
			WS_VSCROLL | ES_READONLY,
			PX_STAT , PY_STAT , WD_STAT , HT_STAT , hwnd , (HMENU)ID_EDIT_ST ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hEditSt, WM_SETFONT, (WPARAM)font12, TRUE);

		hTextCon = CreateWindow(
			TEXT("STATIC") , TEXT("UnConnected") ,
			WS_CHILD | WS_VISIBLE,
			PX_CON , PY_CON, WID_L , HGT_N , hwnd , (HMENU)ID_TXT_CON ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hTextCon, WM_SETFONT, (WPARAM)font24, TRUE);

		// serial# input form
		hEditSer = CreateWindowEx(
			WS_EX_CLIENTEDGE,
			TEXT("EDIT") , TEXT("<SERIAL>") , 
			WS_CHILD | WS_VISIBLE | ES_LEFT | ES_UPPERCASE,
			PX_CON , PY_CON + 30, WID_N , HGT_N , hwnd , (HMENU)ID_EDIT_SER ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		ii = sizeof(char)*8;
		SendMessage(hEditSer, EM_SETLIMITTEXT, (WPARAM)ii, 0);

		hBtnSer = CreateWindow(
			TEXT("BUTTON") , TEXT("Connect") ,
			WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON ,
			PX_CON + WID_N + 10, PY_CON + 30 , WID_N , HGT_N, hwnd , (HMENU)ID_BTN_SER ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		hBtnDcon = CreateWindow(
			TEXT("BUTTON") , TEXT("Disonnect") ,
			WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON ,
			PX_CON + WID_N + 10, PY_CON + 60 , WID_N , HGT_N, hwnd , (HMENU)ID_BTN_DCON ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		EnableWindow(hBtnDcon, FALSE);

		hTextTrg1 = CreateWindow(
			TEXT("STATIC") , TEXT("1st Trigger") ,
			WS_CHILD | WS_VISIBLE,
			PX_TRG , PY_TRG, WID_N , HGT_N , hwnd , NULL ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		hEditTrg1 = CreateWindowEx(
			WS_EX_CLIENTEDGE,
			TEXT("EDIT") , TEXT("hello") , 
			WS_CHILD | WS_VISIBLE | ES_LEFT,
			PX_TRG , PY_TRG + 23, WID_N , HGT_N , hwnd , (HMENU)ID_EDIT_TRG1 ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hEditTrg1, EM_SETLIMITTEXT, (WPARAM)ii, 0);

		hTextTrg2 = CreateWindow(
			TEXT("STATIC") , TEXT("2nd Trigger") ,
			WS_CHILD | WS_VISIBLE,
			PX_TRG + WID_N + 10, PY_TRG, WID_N , HGT_N , hwnd , NULL ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);

		hEditTrg2 = CreateWindowEx(
			WS_EX_CLIENTEDGE,
			TEXT("EDIT") , TEXT("byebye") , 
			WS_CHILD | WS_VISIBLE | ES_LEFT,
			PX_TRG + WID_N + 10, PY_TRG + 23, WID_N , HGT_N , hwnd , (HMENU)ID_EDIT_TRG2 ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hEditTrg2, EM_SETLIMITTEXT, (WPARAM)ii, 0);

		hTextRun = CreateWindow(
			TEXT("STATIC") , TEXT("Stopping") ,
			WS_CHILD | WS_VISIBLE,
			PX_RUN , PY_RUN, WID_L , HGT_N , hwnd , (HMENU)ID_TXT_RUN ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hTextRun, WM_SETFONT, (WPARAM)font24, TRUE);

		hBtnRun = CreateWindow(
			TEXT("BUTTON") , TEXT("Run") ,
			WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON ,
			PX_RUN, PY_RUN + 30 , WID_N , HGT_N, hwnd , (HMENU)ID_BTN_RUN ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		EnableWindow(hBtnRun, FALSE);

		hBtnStop = CreateWindow(
			TEXT("BUTTON") , TEXT("Stop") ,
			WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON ,
			PX_RUN + WID_N + 10, PY_RUN + 30 , WID_N , HGT_N, hwnd , (HMENU)ID_BTN_STOP ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		EnableWindow(hBtnStop, FALSE);

		// listview (main grid)
		hGrid = CreateWindowEx(
			WS_EX_CLIENTEDGE, WC_LISTVIEW, NULL,
			WS_VISIBLE | WS_CHILD | LVS_REPORT | LVS_OWNERDATA,
			PX_GRID, PY_GRID, WD_GRID, HT_GRID,
			hwnd,(HMENU)ID_GRID,((LPCREATESTRUCT)(lp))->hInstance, NULL);

		lvstyle = ListView_GetExtendedListViewStyle(hGrid);
		lvstyle |= LVS_EX_GRIDLINES;
		ListView_SetExtendedListViewStyle(hGrid, lvstyle);
		ii = 0;
		AddColumn(hGrid, 55, "ID", ii++);
		AddColumn(hGrid, 75, "Val", ii++);
		AddColumn(hGrid, 30, "AD", ii++);
		AddColumn(hGrid, 30, "EP", ii++);
		AddColumn(hGrid, 50, "Data", ii++);
		AddColumn(hGrid, 40, "Byte", ii++);
		AddColumn(hGrid, 120, "Data(Hex)", ii++);
		AddColumn(hGrid, 40, "HS", ii++);
		AddColumn(hGrid, 40, "SPD", ii++);
		AddColumn(hGrid, 40, "N", ii++);

		hTextSOF = CreateWindow(
			TEXT("STATIC") , TEXT("LAST SOF : ") ,
			WS_CHILD | WS_VISIBLE,
			PX_SOF , PY_SOF, WID_L , HGT_S , hwnd , (HMENU)ID_TXT_SOF ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hTextSOF, WM_SETFONT, (WPARAM)font12, TRUE);

		hTextTime = CreateWindow(
			TEXT("STATIC") , TEXT("LAST TIMESTAMP : ") ,
			WS_CHILD | WS_VISIBLE,
			PX_SOF + WID_L + 20, PY_SOF, WID_LL , HGT_S , hwnd , (HMENU)ID_TXT_TIME ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		SendMessage(hTextTime, WM_SETFONT, (WPARAM)font12, TRUE);

		// filter selectors
		hChkSOF = CreateWindow(
			TEXT("BUTTON") , TEXT("Ignore SOF") ,
			WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX ,
			PX_FLT , PY_FLT , WID_L , HGT_N , hwnd , (HMENU)ID_CHK_SOF ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		hChkPING = CreateWindow(
			TEXT("BUTTON") , TEXT("Ignore PING") ,
			WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX ,
			PX_FLT , PY_FLT + 30 , WID_L , HGT_N , hwnd , (HMENU)ID_CHK_PING ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		hChkINNAK = CreateWindow(
			TEXT("BUTTON") , TEXT("Ignore INNAK") ,
			WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX ,
			PX_FLT , PY_FLT + 60 , WID_L , HGT_N , hwnd , (HMENU)ID_CHK_INNAK ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);
		hChkSETUP = CreateWindow(
			TEXT("BUTTON") , TEXT("Only SETUP") ,
			WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX ,
			PX_FLT , PY_FLT + 90, WID_L , HGT_N , hwnd , (HMENU)ID_CHK_SETUP ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);

		// initial checks
		SendMessage(hChkPING, BM_SETCHECK, BST_CHECKED, 0);
		SendMessage(hChkINNAK, BM_SETCHECK, BST_CHECKED, 0);

		hBtnSave = CreateWindow(
			TEXT("BUTTON") , TEXT("Export") ,
			WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON ,
			PX_SAVE, PY_SAVE, WID_N , HGT_N, hwnd , (HMENU)ID_BTN_SAVE ,
			((LPCREATESTRUCT)(lp))->hInstance , NULL
			);

		ofn.lStructSize = sizeof (OPENFILENAME);
		ofn.hwndOwner = hWndMain;
		ofn.lpstrFilter = 	TEXT("CSV files (*.csv)\0*.csv\0");
		ofn.lpstrFile = fname;
		ofn.nFilterIndex = 1;
		ofn.nMaxFile = MAX_PATH;
		ofn.lpstrDefExt = TEXT(".csv");
		ofn.Flags = OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT;

		return 0;
		break;
	case WM_SIZE:
		MoveWindow(hGrid, PX_GRID, PY_GRID, WD_GRID, HT_GRID, TRUE);
		break;
	default: 
		break;
	}

	return DefWindowProc(hwnd , msg , wp , lp);
}

///////////////////////////////////////////////////////////
//
// Main function
// 
///////////////////////////////////////////////////////////

int APIENTRY WinMain(
	HINSTANCE hInstance ,
	HINSTANCE hPrevInstance ,
	PSTR lpCmdLine ,
	int nCmdShow ){

		WNDCLASS myc_Main;
		MSG msg;

		myc_Main.style		= CS_HREDRAW | CS_VREDRAW;
		myc_Main.lpfnWndProc	= WndProc;
		myc_Main.cbClsExtra	= myc_Main.cbWndExtra	= 0;
		myc_Main.hInstance		= hInstance;
		myc_Main.hIcon		= LoadIcon(NULL , IDI_APPLICATION);
		myc_Main.hCursor		= LoadCursor(NULL , IDC_ARROW);
		myc_Main.hbrBackground	= (HBRUSH)(COLOR_BTNFACE+1);
		myc_Main.lpszMenuName	= NULL;
		myc_Main.lpszClassName	= TEXT("MYMAIN");

		if (!RegisterClass(&myc_Main)) return 0;

		hWndMain = CreateWindow(
			TEXT("MYMAIN") , TEXT("LEU GUI Sample") ,
			WS_OVERLAPPEDWINDOW ,
			CW_USEDEFAULT , CW_USEDEFAULT , MWIN_WID , MWIN_HGT ,
			NULL , NULL , hInstance , NULL
			);	

		if (hWndMain == NULL) return 0;

		LE650 = new MyLEU(LERUN_TIMES);

		ShowWindow(hWndMain , SW_SHOW);
		UpdateWindow(hWndMain);

		while(GetMessage(&msg , NULL , 0 , 0)){
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		// end
		delete LE650;

		return 0;
}